@@ -97,7 +97,7 @@ public class ServletUtils {
      * @throws IOException
      */
     public static void sendResponse(HttpServletResponse response, int returnCode, String answer) throws IOException {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(returnCode);
         response.getWriter().print(answer);
@@ -159,9 +159,19 @@ public class ServletUtils {
      * @param json JSON object to put into the body with content type application/json
      */
     public static void sendJsonOK(HttpServletResponse response, JsonElement json) {
+        response.setStatus(HttpServletResponse.SC_OK);
+        sendJson(response,json);
+    }
+
+    /**
+     * Send JSON response
+     *
+     * @param response the response to send the 200 code to
+     * @param json JSON object to put into the body with content type application/json
+     */
+    public static void sendJson(HttpServletResponse response, JsonElement json) {
         response.setContentType("application/json; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
-        response.setStatus(HttpServletResponse.SC_OK);
         try {
             response.getWriter().write(json.toString());
         } catch (IOException e) {
@@ -170,7 +180,7 @@ public class ServletUtils {
     }
 
     public static void sendAnswer(HttpServletResponse response, int returnCode, String answer) throws IOException {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(returnCode);
         response.getWriter().print(answer);
@@ -183,7 +193,7 @@ public class ServletUtils {
      * @param answer body text
      */
     public static void sendNotAcceptable(HttpServletResponse response, String answer) {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
         try {
@@ -200,7 +210,7 @@ public class ServletUtils {
      * @param answer body text
      */
     public static void sendError(HttpServletResponse response, String answer) {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=UTF-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
         try {
