@@ -65,6 +65,10 @@ public enum GuardProperty {
      * AES_Key_Length: 256
      */
     aesKeyLength(256),
+    /**
+     * Maximum lifetime of an authentication-token.
+     */
+    authLifeTime("1W"),
     /**
      * Defines how long someone will be locked out after bad attempts.
      * Defaults to 10 minutes
@@ -107,6 +111,7 @@ public enum GuardProperty {
      */
     dbSchemaBaseName("oxguard"),
     defaultLanguage("en_US"),
+    defaultAdvanced(false),
     demo(false),
     /**
      * The amount of hours specifying how long a deleted and exposed key will be
@@ -185,6 +190,16 @@ public enum GuardProperty {
      * details here: http://oxpedia.org/wiki/index.php?title=AppSuite:GuardMailResolver
      */
     mailResolverUrl,
+    /**
+     * Specifies the username for the basic HTTP authentication used for accessing the mail resolver.
+     * If no username is specified here, com.openexchange.guard.restApiUsername will be used as fallback (default)
+     */
+    basicAuthUsername(GuardProperty.EMPTY,GuardProperty.PREFIX + "mailResolverUrl."),
+    /**
+     * Specifies the password for the basic HTTP authentication used for accessing the mail resolver.
+     * If no password is specified here, com.openexchange.guard.restApiPassword will be used as fallback (default)
+     */
+    basicAuthPassword(GuardProperty.EMPTY,GuardProperty.PREFIX + "mailResolverUrl."),    
     /**
      * Max number of connections to same URL (route)
      */
@@ -308,13 +323,27 @@ public enum GuardProperty {
      */
     productName("Guard"),
     /**
-     * Remote Public Key lookup. You can list Public PGP Servers here, preferably
+     * Depreciated.  Now moved to trustedPGP and untrustedPGPDirectory
+     */
+    publicPGPDirectory(""),
+    /**
+     * Remote Public Key lookup. You can list trusted PGP Servers here, preferably
      * servers that have verified keys. Comma separated list. Can specify port
      * https only on port 443.
      * In addition it is possible to add an optional base request path;
      * for example hkp://example.org:11371/custom/path/to/pks/lookup?
      */
-    publicPGPDirectory("hkp://keys.gnupg.net:11371, hkp://pgp.mit.edu:11371"),
+    trustedPGPDirectory(""),
+    /**
+     * Remote Public Key lookup. You can list untrusted Public PGP Servers here.
+     * Keys will be used for sending, but not trusted for verification
+     * Comma separated list. Can specify port
+     * https only on port 443.
+     * In addition it is possible to add an optional base request path;
+     * for example hkp://example.org:11371/custom/path/to/pks/lookup?
+     */
+    untrustedPGPDirectory(""),
+
     /**
      * Specifies the hostname of the Open-Xchange REST API server. The REST API is a
      * service on the Open-Xchange backend. Use 'localhost' in case that the Guard service
