@@ -1,9 +1,9 @@
+
 package com.openexchange.guard.keys;
 
 import java.util.Collection;
-import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import com.openexchange.exception.OXException;
-
+import com.openexchange.guard.keys.internal.hkpclient.RemoteKeyResult;
 
 /**
  * {@link HKPClientService}
@@ -15,19 +15,23 @@ public interface HKPClientService {
 
     /**
      * Tries to find all public PGP keys for the given email
+     * 
      * @param clientToken an identification token put into the X-UI-INTERNAL-ACCOUNT-ID header, or null not not set the header
      * @param email The email to get the public keys for
      * @return A list of public keys found for the given email
      * @throws OXException
      */
-    public Collection<PGPPublicKeyRing> find(String clientToken, String email) throws OXException;
+    public Collection<RemoteKeyResult> find(String clientToken, String email) throws OXException;
 
     /**
-     * Tries to find a public PGP keys for the given email
+     * Tries to find a specific key by ID
+     * 
      * @param clientToken an identification token put into the X-UI-INTERNAL-ACCOUNT-ID header, or null not not set the header
-     * @param email The email to get the public keys for
-     * @return The first found public key ring for the given email
+     * @param id The ID of the key
+     * @param email (optional) Email address of the sender. If null, no SRV lookup
+     * @return The public key for the given id
      * @throws OXException
      */
-    public PGPPublicKeyRing findFirst(String clientToken, String email) throws OXException;
+    public RemoteKeyResult find(String clientToken, Long id, String email) throws OXException;
+
 }
