@@ -51,11 +51,11 @@ package com.openexchange.guard.keys.internal;
 
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.List;
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import com.google.gson.JsonObject;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.cipher.GuardCipherAlgorithm;
 import com.openexchange.guard.cipher.GuardCipherFactoryService;
@@ -74,12 +74,11 @@ import com.openexchange.guard.keys.dao.GuardKeys;
 import com.openexchange.guard.keys.dao.OGPGPKey;
 import com.openexchange.guard.keys.dao.RecipKey;
 import com.openexchange.guard.keys.exceptions.PublicKeyExceptionCodes;
+import com.openexchange.guard.keys.internal.hkpclient.RemoteKeyResult;
 import com.openexchange.guard.keys.osgi.Services;
 import com.openexchange.guard.keys.storage.KeyTableStorage;
 import com.openexchange.guard.keys.storage.OGPGPKeysStorage;
 import com.openexchange.guard.keys.util.PGPUtil;
-import com.openexchange.guard.mailcreator.MailCreatorService;
-import com.openexchange.guard.notification.GuardNotificationService;
 import com.openexchange.guard.oxapi.Capabilities;
 import com.openexchange.guard.ratifier.GuardRatifierService;
 import com.openexchange.guard.translation.GuardTranslationService;
@@ -182,9 +181,9 @@ public class PublicKeyServiceImpl implements PublicKeyService {
                         if (send) {
                             LOG.debug("Found id for " + email + " in main ox users, creating key");
 
-                            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+                            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
                             result.setNewGuestPass(cipherService.generateRandomPassword());
-
+/*
                             MailCreatorService mailCreatorService = Services.getService(MailCreatorService.class);
 
                             GuardConfigurationService guardConfigService = Services.getService(GuardConfigurationService.class);
@@ -194,7 +193,7 @@ public class PublicKeyServiceImpl implements PublicKeyService {
 
                             GuardNotificationService guardNotificationService = Services.getService(GuardNotificationService.class);
                             guardNotificationService.send(pwEmail, null);
-
+*/
                             KeyTableStorage ogKeyTableStorage = Services.getService(KeyTableStorage.class);
                             GuardKeys currentKeyForUser = ogKeyTableStorage.getCurrentKeyForUser(result.getUserid(), result.getCid());
                             boolean markNewKeyAsCurrent = true;
@@ -228,16 +227,23 @@ public class PublicKeyServiceImpl implements PublicKeyService {
             } catch (Exception e) {
                 throw PublicKeyExceptionCodes.CRYPTO_ERROR.create(e, e.getMessage());
             }
+            
+            //Find via HKP
             HKPClientService hkpClient = Services.getService(HKPClientService.class);
-            PGPPublicKeyRing remoteKey = hkpClient.findFirst(hkpClientToken, email);
-            if (remoteKey != null) {
-                result = new RecipKey();
-                result.setPgpring(remoteKey);
-                result.setEmail(email);
-                result.setPgp(true);
-                result.setExpired(PGPUtil.checkAllExpired(remoteKey));
+            PGPPublicKeyRing remoteKey = null;
+            Collection<RemoteKeyResult> results = hkpClient.find(hkpClientToken, email);
+            if(results != null && !results.isEmpty()) {
+                //Use the first key found
+                remoteKey = results.iterator().next().getRing();
+                if (remoteKey != null) {
+                    result = new RecipKey();
+                    result.setPgpring(remoteKey);
+                    result.setEmail(email);
+                    result.setPgp(true);
+                    result.setExpired(PGPUtil.checkAllExpired(remoteKey));
+                }
             }
-
+            
             if (result != null) {
                 return result;
             }
@@ -291,7 +297,7 @@ public class PublicKeyServiceImpl implements PublicKeyService {
 
     /**
      * Checks if the user has Webmail capability enabled
-     * 
+     *
      * @param id the user identifier
      * @param cid the context identifier
      * @return true if the specified user has the "webmail" capability; false otherwise
@@ -337,7 +343,7 @@ public class PublicKeyServiceImpl implements PublicKeyService {
                 GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
                 guestlang = configService.getProperty(GuardProperty.defaultLanguage);
             }
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
             result.setNewGuestPass(cipherService.generateRandomPassword());
 
             GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
