@@ -281,11 +281,6 @@ public class GuardKeyServiceImpl implements GuardKeyService {
         if (Strings.isEmpty(password)) {
             return "-1";
         }
-        GuardConfigurationService configService = Services.getService(GuardConfigurationService.class);
-        if (configService.getBooleanProperty(GuardProperty.noRecovery)) {
-            // If selected for no recovery, then just return blank.
-            return "";
-        }
         password = CipherUtil.getSHA(password, keys.getSalt());
         return encryptRecoveryHash(keys, password);
     }
@@ -297,16 +292,6 @@ public class GuardKeyServiceImpl implements GuardKeyService {
      */
     @Override
     public GuardKeys generateKeyPair(String name, String email, String password) throws OXException {
-        return generateKeyPair(name, email, password, false);
-    }
-
-    /*
-     * (non-Javadoc)
-     *
-     * @see com.openexchange.guard.keys.GuardKeyService#generateKeyPair(java.lang.String, java.lang.String, java.lang.String, boolean)
-     */
-    @Override
-    public GuardKeys generateKeyPair(String name, String email, String password, boolean failover) throws OXException {
         try {
             GuardConfigurationService guardConfigService = Services.getService(GuardConfigurationService.class);
 
@@ -317,16 +302,17 @@ public class GuardKeyServiceImpl implements GuardKeyService {
             // Load pre-generated key pairs
             AsymmetricCipherKeyPair mst = getKeyPair();
             if (mst == null) {
-                return null;
+                LOG.info("No pre-generated key pair for master key available. Generate on-the-flow ...");
+                return generateKeys(name, email, password);
             }
-            AsymmetricCipherKeyPair enc = getKeyPair();
-            if (enc == null) {
-                return null;
-            }
-
             // Create OGPGPKey Pairs
             PGPKeyPair rsa_master = new BcPGPKeyPair(PGPPublicKey.RSA_GENERAL, mst, new Date());
 
+            AsymmetricCipherKeyPair enc = getKeyPair();
+            if (enc == null) {
+                LOG.info("No pre-generated key pair for sub key available. Generate on-the-flow ...");
+                return generateKeys(name, email, password);
+            }
             // SubKey
             PGPKeyPair rsa_encrypt = new BcPGPKeyPair(PGPPublicKey.RSA_GENERAL, enc, new Date());
 
@@ -501,13 +487,13 @@ public class GuardKeyServiceImpl implements GuardKeyService {
                 GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
                 GuardCipherService rsaCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.RSA);
                 GuardKeys masterKeys = ogKeyTableStorage.getGuardDaemonKey();
-                PrivateKey privateKey = decryptPrivateKey(masterKeys.getEncodedPrivate(), masterKeyService.getMasterKey().getMC(), masterKeys.getSalt(), true);
+                PrivateKey privateKey = decryptPrivateKey(masterKeys.getEncodedPrivate(), masterKeyService.getMasterKey().getMC(), masterKeys.getSalt());
                 String encrhash = rsaCipherService.decrypt(currentKey.getRecovery(), privateKey);
                 if (Strings.isEmpty(encrhash)) {
                     LOG.error("Unable to decode master key.  Possible corrupted data");
                     throw new Exception("Bad master key");
                 }
-                GuardCipherService aesCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+                GuardCipherService aesCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
                 String rsapub = currentKey.getEncodedPublic();
                 hash = aesCipherService.decrypt(encrhash, masterKeyService.getMasterKey().getRC() + (rsapub == null ? "" : Integer.toString(rsapub.hashCode())), currentKey.getSalt());
             }
@@ -522,7 +508,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
     public boolean isRecoveryEnabled(int userId, int contextId) throws OXException {
         KeyTableStorage ogKeyTableStorage = Services.getService(KeyTableStorage.class);
         GuardKeys currentKey = ogKeyTableStorage.getCurrentKeyForUser(userId, contextId);
-        
+
         if (currentKey != null && !Strings.isEmpty(currentKey.getRecovery())) {
             return true;
         }
@@ -531,7 +517,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
 
     @Override
     public String generatePassword() throws OXException {
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         return cipherService.generateRandomPassword();
     }
 
@@ -559,6 +545,26 @@ public class GuardKeyServiceImpl implements GuardKeyService {
         }
     }
 
+    @Override
+    public GuardKeys changeTemporaryPassword (int userid, int cid, String newpass) throws OXException {
+    	GuardKeys oldkeys = getKeys(userid, cid);
+    	if (oldkeys == null) {
+    		LOG.error("Unable to retrieve keys for " + userid + " cid " + cid);
+    		throw KeysExceptionCodes.NO_KEY_ERROR.create();
+    	}
+    	if (oldkeys.getLastup() != null || oldkeys.getQuestion() != null) {  // Only allowed for first use
+    		LOG.error("Unable to change temporary password for %d cid %d due to already updated", userid, cid);
+    		throw KeysExceptionCodes.PREVIOUSLY_CHANGED.create();
+    	}
+    	String hash = recoverPasswordHash(oldkeys.getUserid(), oldkeys.getContextid());
+    	if (hash.equals("")) {
+            throw KeysExceptionCodes.NO_RECOVERY_ERROR.create();
+        }
+    	GuardKeys newkeys = changePasswordWithRecovery(hash, newpass, oldkeys);
+        return (newkeys);
+
+    }
+
     /*
      * (non-Javadoc)
      *
@@ -567,7 +573,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
     @Override
     public GuardKeys changePassword(String oldpass, String newpass, GuardKeys oldKey) throws OXException {
         if (!oldKey.privKeyNull()) {
-            PrivateKey priv = decryptPrivateKey(oldKey.getEncodedPrivate(), oldpass, oldKey.getSalt(), true);
+            PrivateKey priv = decryptPrivateKey(oldKey.getEncodedPrivate(), oldpass, oldKey.getSalt());
             if (priv == null) {
                 return null;
             }
@@ -599,7 +605,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
     //FIXME: double code with changePassword, remove that
     public GuardKeys changePasswordWithRecovery(String oldPassHash, String newPass, GuardKeys oldKey) throws OXException {
         if (!oldKey.privKeyNull()) {
-            PrivateKey priv = decryptPrivateKeyWithHash(oldKey.getEncodedPrivate(), oldPassHash, oldKey.getSalt(), true);
+            PrivateKey priv = decryptPrivateKeyWithHash(oldKey.getEncodedPrivate(), oldPassHash, oldKey.getSalt());
             if (priv == null) {
                 LOG.error("Failed to change password");
                 throw KeysExceptionCodes.NO_KEY_ERROR.create(oldKey.getEmail());
@@ -630,9 +636,9 @@ public class GuardKeyServiceImpl implements GuardKeyService {
      */
     @Override
     public boolean storeQuestion(int id, int cid, String question, String answer) throws OXException {
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         String encranswer = null;
-        if (answer != null) {
+        if (answer != null && !question.equals("")) {
             GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
             encranswer = cipherService.encrypt(answer, masterKeyService.getMasterKey().getRC(), question);
         }
@@ -653,7 +659,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
      */
     @Override
     public String encryptPrivateKey(PrivateKey privateKey, String password, String salt) throws OXException {
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         return cipherService.encrypt(Base64.encodeBase64String(privateKey.getEncoded()), CipherUtil.getSHA(password, salt), salt);
     }
 
@@ -663,16 +669,16 @@ public class GuardKeyServiceImpl implements GuardKeyService {
      * @see com.openexchange.guard.keys.GuardKeyService#decryptPrivateKey(java.lang.String, java.lang.String, java.lang.String, boolean)
      */
     @Override
-    public PrivateKey decryptPrivateKey(String privateKey, String password, String salt, boolean logBadAttempt) throws OXException {
-        return decryptPrivateKeyWithHash(privateKey, CipherUtil.getSHA(password, salt), salt, logBadAttempt);
+    public PrivateKey decryptPrivateKey(String privateKey, String password, String salt) throws OXException {
+        return decryptPrivateKeyWithHash(privateKey, CipherUtil.getSHA(password, salt), salt);
     }
 
     @Override
-    public PrivateKey decryptPrivateKeyWithHash(String privateKey, String password, String salt, boolean logBadAttempt) throws OXException {
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+    public PrivateKey decryptPrivateKeyWithHash(String privateKey, String password, String salt) throws OXException {
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         byte[] decoded;
         try {
-            decoded = cipherService.decrypt(privateKey, password, salt, logBadAttempt).getBytes("UTF-8");
+            decoded = cipherService.decrypt(privateKey, password, salt).getBytes("UTF-8");
             if (decoded.length == 0) {
                 throw GuardAuthExceptionCodes.BAD_PASSWORD_DECRYPTION.create();
             }
@@ -708,7 +714,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
 
         String recovery = "";
         if (guardDaemonKey != null) {
-            GuardCipherService aesCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            GuardCipherService aesCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
             GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
             String encrPass = aesCipherService.encrypt(password, masterKeyService.getMasterKey().getRC() + (keys.getPublicKey() == null ? "" : Integer.toString(keys.getEncodedPublic().hashCode())), keys.getSalt());
             GuardCipherService rsaCipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.RSA);
@@ -742,7 +748,7 @@ public class GuardKeyServiceImpl implements GuardKeyService {
         }
 
         GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         data = cipherService.decrypt(data, masterKeyService.getMasterKey().getRC(), "key");
         if (Strings.isEmpty(data)) {
             LOG.error("Unable to decode pre-compiled keys.  Password file change??");
