@@ -54,9 +54,12 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Comparator;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
+import java.util.TreeSet;
 import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.bouncycastle.bcpg.sig.RevocationReasonTags;
 import org.bouncycastle.openpgp.PGPEncryptedData;
@@ -116,7 +119,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#updatePGPKeys(int, int, com.openexchange.guard.keys.dao.GuardKeys)
      */
     @Override
@@ -129,7 +132,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPublicKeyRingById(long, int, int)
      */
     @Override
@@ -155,7 +158,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPublicKeyRingById(java.lang.String, int, int)
      */
     @Override
@@ -180,20 +183,34 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPublicKeyRing(java.lang.String)
      */
     @Override
     public List<PGPPublicKeyRing> getPublicKeyRing(String email) throws OXException {
         GuardKeyService keyService = Services.getService(GuardKeyService.class);
         PGPKeysStorage pgpKeysStorage = Services.getService(PGPKeysStorage.class);
-
         List<PGPKeys> internalKeyMappings = pgpKeysStorage.getInternalByEmail(email);
+
+        //remove duplicate mappings
+        Set<PGPKeys> duplicatesRemoved = new TreeSet<PGPKeys>(new Comparator<PGPKeys>() {
+
+            @Override
+            public int compare(PGPKeys key1, PGPKeys key2) {
+                if (key1.getKeyId() == key2.getKeyId()) {
+                    return 0;
+                }
+                return 1;
+            }
+        });
+        duplicatesRemoved.addAll(internalKeyMappings);
+        internalKeyMappings = new ArrayList<PGPKeys>(duplicatesRemoved);
+
         List<PGPPublicKeyRing> ret = new ArrayList<PGPPublicKeyRing>();
         for (PGPKeys internalMapping : internalKeyMappings) {
             if (internalMapping.isLocal()) {
-                GuardKeys key = keyService.getKeysFromEmail(email);
-                if(key != null){
+                GuardKeys key = keyService.getKeysFromEmail(email,internalMapping.getKeyId());
+                if (key != null) {
                     ret.add(key.getPublicKeyRing());
                 }
             }
@@ -203,7 +220,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#updateDuplicatePGPKey(int, int, com.openexchange.guard.keys.dao.GuardKeys, java.lang.String, java.lang.Long)
      */
     @Override
@@ -214,7 +231,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#export(org.bouncycastle.openpgp.PGPPublicKeyRing)
      */
     @Override
@@ -243,7 +260,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPGPPrivateKey(java.lang.String, java.lang.String, org.bouncycastle.openpgp.PGPSecretKeyRing)
      */
     @Override
@@ -268,7 +285,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPGPPrivateKey(java.lang.String, java.lang.String, org.bouncycastle.openpgp.PGPSecretKeyRing, long)
      */
     @Override
@@ -284,13 +301,13 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#getPGPSignKey(org.bouncycastle.openpgp.PGPSecretKeyRing, java.lang.String, java.lang.String)
      */
     @Override
     public PGPPrivateKey getPGPSignKey(PGPSecretKeyRing keyRing, String password, String salt) throws OXException {
         PGPPrivateKey signingKey = getDecodedPrivate(PGPUtil.getSigningKey(keyRing), password, salt);
-        if(signingKey == null) {
+        if (signingKey == null) {
             LOG.error("Unable to find PGP signing key");
         }
         return signingKey;
@@ -298,7 +315,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#checkSignatures(int, int, org.bouncycastle.openpgp.PGPPublicKeyRing)
      */
     @Override
@@ -314,7 +331,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#checkSignatures(int, int, org.bouncycastle.openpgp.PGPPublicKey)
      */
     @SuppressWarnings("unchecked")
@@ -331,7 +348,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#addUID(org.bouncycastle.openpgp.PGPPrivateKey, org.bouncycastle.openpgp.PGPPublicKeyRing, java.lang.String)
      */
     @Override
@@ -354,7 +371,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#checkBadUID(com.openexchange.guard.keys.dao.GuardKeys, java.lang.String)
      */
     @SuppressWarnings("unchecked")
@@ -393,7 +410,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#createPublicKeyRingFromPrivateKey(org.bouncycastle.openpgp.PGPSecretKeyRing)
      */
     @Override
@@ -421,7 +438,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#revokePGPPublicKeys(com.openexchange.guard.keys.dao.GuardKeys, java.lang.String, java.lang.String)
      */
     @Override
@@ -467,7 +484,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#signPGPPublicKey(org.bouncycastle.openpgp.PGPPublicKey, com.openexchange.guard.keys.dao.GuardKeys, java.lang.String)
      */
     @Override
@@ -487,7 +504,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#signPGPPublicKeyRing(org.bouncycastle.openpgp.PGPSecretKey, org.bouncycastle.openpgp.PGPPrivateKey, org.bouncycastle.openpgp.PGPPublicKey)
      */
     @Override
@@ -510,7 +527,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#exportPGPPrivateKey(com.openexchange.guard.keys.dao.GuardKeys, java.lang.String)
      */
     @Override
@@ -732,7 +749,7 @@ public class PGPKeyServiceImpl implements PGPKeyService {
 
     /*
      * (non-Javadoc)
-     *
+     * 
      * @see com.openexchange.guard.keys.PGPKeyService#importPGPPrivateKey(com.openexchange.guard.keys.dao.GuardKeys, java.lang.String, java.lang.String, java.lang.String)
      */
     @Override
