@@ -860,6 +860,49 @@ public class KeyTableStorageImpl implements KeyTableStorage {
         }
         return key;
     }
+    
+
+    @Override
+    public GuardKeys getKeyForGuest(String email, int shardId) throws OXException {
+        GuardDatabaseService guardDatabaseService = Services.getService(GuardDatabaseService.class);
+
+        Connection connection = guardDatabaseService.getReadOnlyForShard(shardId);
+
+        PreparedStatement stmt = null;
+        ResultSet resultSet = null;
+        try {
+            stmt = connection.prepareStatement(KeyTableSql.SELECT_FOR_EMAIL_IN_CONTEXT_STMT);
+            stmt.setString(1, email);
+
+            resultSet = stmt.executeQuery();
+            if (resultSet.next()) {
+                return createKey(resultSet);
+            }
+            return null;
+        } catch (SQLException e) {
+            throw GuardKeyTableStorageExceptionCodes.SQL_ERROR.create(e, e.getMessage());
+        } finally {
+            DBUtils.closeSQLStuff(resultSet, stmt);
+            guardDatabaseService.backReadOnlyForShard(shardId, connection);
+        }
+    }
+
+    /* (non-Javadoc)
+     * @see com.openexchange.guard.keys.storage.KeyTableStorage#exists()
+     */
+    @Override
+    public boolean exists(int userId, int contextId, int shardId) throws OXException {
+        GuardDatabaseService guardDatabaseService = Services.getService(GuardDatabaseService.class);
+        GuardConnectionWrapper readOnly = guardDatabaseService.getReadOnly(userId, contextId, shardId);
+        try {
+            return DBUtils.tableExists(readOnly.getConnection(), KeyTableSql.OG_KEY_TABLE);
+        } catch (SQLException e) {
+            throw GuardKeyTableStorageExceptionCodes.SQL_ERROR.create(e, e.getMessage());
+        }
+        finally {
+            guardDatabaseService.backReadOnly(readOnly);
+        }
+    }
 
     /////////////////////////////////// HELPERS //////////////////////////////////
 
@@ -925,28 +968,4 @@ public class KeyTableStorageImpl implements KeyTableStorage {
         return json;
     }
 
-    @Override
-    public GuardKeys getKeyForGuest(String email, int shardId) throws OXException {
-        GuardDatabaseService guardDatabaseService = Services.getService(GuardDatabaseService.class);
-
-        Connection connection = guardDatabaseService.getReadOnlyForShard(shardId);
-
-        PreparedStatement stmt = null;
-        ResultSet resultSet = null;
-        try {
-            stmt = connection.prepareStatement(KeyTableSql.SELECT_FOR_EMAIL_IN_CONTEXT_STMT);
-            stmt.setString(1, email);
-
-            resultSet = stmt.executeQuery();
-            if (resultSet.next()) {
-                return createKey(resultSet);
-            }
-            return null;
-        } catch (SQLException e) {
-            throw GuardKeyTableStorageExceptionCodes.SQL_ERROR.create(e, e.getMessage());
-        } finally {
-            DBUtils.closeSQLStuff(resultSet, stmt);
-            guardDatabaseService.backReadOnlyForShard(shardId, connection);
-        }
-    }
 }
