@@ -62,6 +62,7 @@ import com.google.gson.JsonPrimitive;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.common.java.Strings;
 import com.openexchange.guard.configuration.GuardConfigurationService;
+import com.openexchange.guard.configuration.GuardProductName;
 import com.openexchange.guard.configuration.GuardProperty;
 import com.openexchange.guard.mailcreator.MailCreatorService;
 import com.openexchange.guard.translation.ConditionalVissibleTemplateTransformation;
@@ -165,11 +166,7 @@ public class MailCreatorServiceImpl implements MailCreatorService {
             to = to + ", ";
         }
         if (name.length() > 0) {
-            if (name.startsWith("\"")) {
                 to = to + name + "<" + email + ">";
-            } else {
-                to = to + "\"" + name + "\"<" + email + ">";
-            }
         } else {
             to = to + email;
         }
@@ -214,7 +211,7 @@ public class MailCreatorServiceImpl implements MailCreatorService {
     public JsonObject addFrom(JsonObject mail, String name, String email) {
         String from = email;
         if (name != null && name.length() > 0) {
-            from = "\"" + name + "\"<" + email + ">";
+            from = name + "<" + email + ">";
         }
         mail.getAsJsonObject("data").add("from", new JsonPrimitive(from));
         return (mail);
@@ -370,8 +367,7 @@ public class MailCreatorServiceImpl implements MailCreatorService {
      * @throws OXException
      */
     public String setProductName (String content, String host, int id, int cid) throws OXException {
-    	GuardConfigurationService guardConfigService = Services.getService(GuardConfigurationService.class);
-    	String name = guardConfigService.getProductName(host, id, cid);
+    	String name = GuardProductName.getProductName(host, cid, id);
     	content = content.replace("$productName", name);
     	return (content);
     }
