@@ -285,14 +285,8 @@ public class GuardNotificationServiceImpl implements GuardNotificationService {
 
             // Connect and send
             {
-                final GuardConfigurationService guardConfigService = Services.getService(GuardConfigurationService.class);
-                final String server = guardConfigService.getProperty(GuardProperty.guestSMTPServer);
-                final int port = guardConfigService.getIntProperty(GuardProperty.guestSMTPPort);
-                final String username = guardConfigService.getProperty(GuardProperty.guestSMTPUsername);
-                final String password = guardConfigService.getProperty(GuardProperty.guestSMTPPassword);
-
                 final Transport transport = session.getTransport("smtp");
-                transport.connect(server, port, username, password);
+                transport.connect();
                 transport.sendMessage(message, message.getAllRecipients());
                 transport.close();
             }
@@ -318,9 +312,8 @@ public class GuardNotificationServiceImpl implements GuardNotificationService {
                 fromaddr.setAddress(sender);
                 msg.setFrom(fromaddr);
             }
-            final GuardConfigurationService guardConfigService = Services.getService(GuardConfigurationService.class);
             transport = session.getTransport("smtp");
-            transport.connect(guardConfigService.getProperty(GuardProperty.guestSMTPServer), guardConfigService.getIntProperty(GuardProperty.guestSMTPPort), guardConfigService.getProperty(GuardProperty.guestSMTPUsername), guardConfigService.getProperty(GuardProperty.guestSMTPPassword));
+            transport.connect();
             transport.sendMessage(msg, msg.getAllRecipients());
             transport.close();
         } catch (final MessagingException | UnsupportedEncodingException e) {
