@@ -96,6 +96,6 @@ public class GuardRatifierServiceImpl implements GuardRatifierService {
 
     @Override
     public String normalise(String data) throws OXException {
-        return Normalizer.normalizeString(data);
+        return new Normalizer().normalizeString(data);
     }
 }
