@@ -79,6 +79,7 @@ import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import com.openexchange.exception.OXException;
+import com.openexchange.guard.common.java.Strings;
 import com.openexchange.guard.configuration.GuardConfigurationService;
 import com.openexchange.guard.configuration.GuardProperty;
 import com.openexchange.guard.translation.GuardTranslationService;
@@ -132,13 +133,13 @@ public class GuardTranslationServiceImpl implements GuardTranslationService {
         try {
             // Read the languages file
             FileInputStream inputStream = new FileInputStream(l10nDirectory + File.separator + "languages.xml");
-            String xmlfile = IOUtils.toString(inputStream);
+            String xmlfile = IOUtils.toString(inputStream, "UTF-8");
             inputStream.close();
 
             // Build the XML document
             DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
             DocumentBuilder builder = factory.newDocumentBuilder();
-            Document doc = builder.parse(new InputSource(new ByteArrayInputStream(xmlfile.getBytes("utf-8"))));
+            Document doc = builder.parse(new InputSource(new ByteArrayInputStream(xmlfile.getBytes("UTF-8"))));
             doc.getDocumentElement().normalize();
 
             // Read all available languages
@@ -160,7 +161,7 @@ public class GuardTranslationServiceImpl implements GuardTranslationService {
                             found = true;
                             // Load the actual PO file, a.k.a. the translation
                             loadPOFile(mapping.getAbsolutePath(), id);
-                		}
+                        }
                     }
                     if (found) {
                         // Load the translations of the available languages
@@ -294,8 +295,9 @@ public class GuardTranslationServiceImpl implements GuardTranslationService {
         if (l10nDirectory.endsWith("/")) {
             l10nDirectory = l10nDirectory.substring(0, l10nDirectory.length() - 1);
         }
-        if (language == null || language.equals(""))
+        if (language == null || language.equals("")) {
             language = defaultCode;
+        }
         String filename = l10nDirectory + File.separator + HELP_ATTACHMENT_FOLDER + File.separator + HELP_ATTACHMENT_PREFIX + language + ".txt";
         File f = new File(filename);
         if (f.exists()) {
@@ -367,7 +369,11 @@ public class GuardTranslationServiceImpl implements GuardTranslationService {
             if (languageMap != null) {
                 Translation trans = languageMap.get(text);
                 if (trans != null) {
-                    return trans.getTranslation();
+                    String translation = trans.getTranslation();
+                    if (Strings.isNotEmpty(translation)) { // only if translation is available. Ignore empty ones.
+                        return translation;
+                    }
+                    return text;
                 }
             }
             return text;
