@@ -191,7 +191,7 @@ public class FileHandler {
         String folderId = ServletUtils.getStringParameter(request, "folder", true);
         String version = ServletUtils.getStringParameter(request, "version");
         String auth = ServletUtils.getStringParameter(request, "auth");
-        String lang = ServletUtils.getStringParameter(request, "lang");
+        String lang = ServletUtils.getStringParameter(request, "language");
         String fileName = ServletUtils.getStringParameter(request, "filename", true);
         boolean download = ServletUtils.getBooleanParameter(request, "download", false);
 
@@ -264,7 +264,7 @@ public class FileHandler {
      * @param answer String to put in HTML
      */
     private void sendOK(HttpServletResponse response, String answer) {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=utf-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_OK);
         try {
@@ -275,7 +275,7 @@ public class FileHandler {
     }
 
     private void sendFail(HttpServletResponse response, String answer) {
-        response.setContentType("text/html");
+        response.setContentType("text/html; charset=utf-8");
         response.addHeader("Access-Control-Allow-Origin", "*");
         response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
         try {
@@ -499,5 +499,4 @@ public class FileHandler {
         JsonArray permissions = jdata.get("permissions").getAsJsonArray();
         return (permissions);
     }
-
 }
