@@ -50,6 +50,7 @@
 package com.openexchange.guard.internal;
 
 import java.io.IOException;
+import java.util.Date;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicReference;
@@ -72,6 +73,7 @@ import com.openexchange.guard.common.servlets.utils.ServletUtils;
 import com.openexchange.guard.common.util.CipherUtil;
 import com.openexchange.guard.common.util.JsonUtil;
 import com.openexchange.guard.configuration.GuardConfigurationService;
+import com.openexchange.guard.configuration.GuardProductName;
 import com.openexchange.guard.configuration.GuardProperty;
 import com.openexchange.guard.email.storage.ogEmail.EmailStorage;
 import com.openexchange.guard.exceptions.GuardAuthExceptionCodes;
@@ -127,6 +129,7 @@ public class Auth {
             if (json.get("name") != null) {
                 name = json.get("name").getAsString();
             }
+            name =  sanitize(name);
 
             GuardMasterKeyService masterKeyService = Services.getService(GuardMasterKeyService.class);
             keys = masterKeyService.createMasterKeys(userid, cid, email, name, password, language, true, GuardSettings.checkCreateRecovery(cookie, request), true);// Flag true for user created
@@ -151,12 +154,16 @@ public class Auth {
                 if (json.get("name") != null) {
                     name = json.get("name").getAsString();
                 }
+                name = sanitize(name);
 
                 GuardKeys newkey = keyService.generateKeyPair(name, email, password);
                 newkey.setLanguage(language);
                 newkey.setContextid(keys.getContextid());
                 newkey.setUserid(keys.getUserid());
-                String recovery = keyService.createRecovery(newkey, password);
+                String recovery = "";
+                if (GuardSettings.checkCreateRecovery(cookie, request)) {
+                	recovery = keyService.createRecovery(newkey, password);
+                }
                 //acc.addNewKey(newkey, keys, recovery);
                 newkey.setRecovery(recovery);
                 keyService.addNewKey(newkey);
@@ -171,6 +178,11 @@ public class Auth {
         }
     }
 
+    private String sanitize(String data) throws OXException {
+        GuardRatifierService ratifierService = Services.getService(GuardRatifierService.class);
+        return ratifierService.normalise(data);
+    }
+
     /**
      * Change password
      *
@@ -205,8 +217,10 @@ public class Auth {
                 answer = json.get("answer").getAsString();
             }
             if (json.has("email")) {
-                question = "e";
                 answer = json.get("email").getAsString();
+                if (!answer.equals("")) {
+                    question = "e";
+                }
             }
             // if ox member, verify actually logged in to ui
             if (cid > 0) {
@@ -229,8 +243,12 @@ public class Auth {
                 return (null);
             }
             String oldpass = "";
-            oldpass = JsonUtil.getStringFromJson(json, "oldpass", true).trim();
-            keys = keyService.changePassword(oldpass, newpass, keys);
+            oldpass = JsonUtil.getStringFromJson(json, "oldpass", true);
+            if (oldpass == null || oldpass.equals("")) {
+            	keys = keyService.changeTemporaryPassword(userid, cid, newpass);
+            } else {
+            	keys = keyService.changePassword(oldpass.trim(), newpass, keys);
+            }
             if (keys == null) {
                 LOG.info("Bad password for change");
                 response.getWriter().write("{\"auth\":\"Bad password\"}");
@@ -243,13 +261,19 @@ public class Auth {
                     recoveryAvail = true;
                     recovery = keyService.createRecovery(keys, newpass);
                 }
+                Date now = new Date();
+                keys.setLastup(new java.sql.Date(now.getTime()));
                 keyService.updateKeys(userid, cid, keys, recovery, false);
-                if (!answer.equals("")) {// If sent with question and answer, then store
-                    if (question.equals("")) {
-                        question = "default";
-                    }
-                    keyService.storeQuestion(userid, cid, question, answer);
+
+                if (cid < 0) {  // Guests have question answer to store
+                	if (question.equals("") && !answer.equals("")) {
+                		question = "default";
+                	}
+                	if (answer != null && !answer.equals("")) {
+                		keyService.storeQuestion(userid, cid, question, answer);
+                	}
                 }
+
                 // String sessionID = json.get("sessionID").getAsString();
                 GuardSessionService sessionService = Services.getService(GuardSessionService.class);
                 String token = sessionService.getToken(cookie.getJSESSIONID());
@@ -257,7 +281,7 @@ public class Auth {
                     token = sessionService.newToken(cookie.getJSESSIONID(), userid, cid);
                 }
                 json.add("encr_password", new JsonPrimitive(newpass));
-                GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+                GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_GCM);
                 JsonObject returndata = new JsonObject();
                 returndata.addProperty("auth", cipherService.encrypt(json.toString(), token));
                 returndata.addProperty("recovery", recoveryAvail);
@@ -318,9 +342,12 @@ public class Auth {
         List<GuardKeys> userKeys = ogKeyTableStorage.getKeysForUser(userid, cid);
         if (userKeys.size() > 0) {
             String encr_answer = userKeys.get(0).getAnswer();
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            if (encr_answer == null) {
+                return null;
+            }
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
             String rc = Services.getService(GuardMasterKeyService.class).getMasterKey().getRC();
-            return cipherService.decrypt(encr_answer, rc, "e", false);
+            return cipherService.decrypt(encr_answer, rc, "e");
         }
         return null;
     }
@@ -382,18 +409,18 @@ public class Auth {
             String encr_answer = key.getAnswer();
             String userEmail = key.getEmail();
             boolean secondary = true;
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
             // Mailing to secondary email address if web.  If from command line, then email to primary
             String rc = Services.getService(GuardMasterKeyService.class).getMasterKey().getRC();
-            String secondEmail = web ? cipherService.decrypt(encr_answer, rc, "e", false) : userEmail;
+            String secondEmail = web ? cipherService.decrypt(encr_answer, rc, "e") : userEmail;
             if ((secondEmail == null) || secondEmail.equals("") || secondEmail.equals(userEmail)) {
                 secondEmail = userEmail;
                 secondary = false;
             }
             GuardRatifierService validatorService = Services.getService(GuardRatifierService.class);
             validatorService.validate(secondEmail);
-
-            reset(userid, cid, response, web, templid, lang, userEmail, secondEmail, request.getServerName());
+            String hostname = (request == null ? "" : request.getServerName());
+            reset(userid, cid, response, web, templid, lang, userEmail, secondEmail, hostname);
 
             if (web) {
                 GuardBadStorage badStorage = Services.getService(GuardBadStorage.class);
@@ -458,6 +485,9 @@ public class Auth {
         cookie.setPath("/");
         response.addCookie(cookie);
         String username = JsonUtil.getStringFromJson(login_json, "username", true);
+        if(username.isEmpty()) {
+            throw GuardCoreExceptionCodes.JSON_PARAMETER_MISSING.create(username);
+        }
         String pin = login_json.get("pin") == null ? null : login_json.get("pin").getAsString();
         try {
             GuardBadStorage badStorage = Services.getService(GuardBadStorage.class);
@@ -490,12 +520,18 @@ public class Auth {
             json.addProperty("sessionID", sessionID);
             json.addProperty("cid", keys.getContextid());
             json.addProperty("recoveryAvail", keys.isRecoveryAvail());
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_GCM);
             String returndata = "Bad Password";
 
             if (!PGPUtils.verifyPassword(password, keys.getSalt(), keys.getPGPSecretKeyRing())) {
                 badStorage.addBad(username);
                 LOG.info("Bad password for " + username);
+                JsonObject sendjson = new JsonObject();
+                sendjson.addProperty("auth", returndata);
+                sendjson.addProperty("recoveryAvail", keys.isRecoveryAvail());
+                ServletUtils.sendJsonOK(response, sendjson);
+                return;
             } else {
                 // Check if PIN
                 if ((keys.getLastup() == null) && (keys.getQuestion() == null ? false : keys.getQuestion().equals("PIN"))) {
@@ -517,13 +553,14 @@ public class Auth {
 
             // The json returned in plain text.  no password here (encrypted within auth)  Include settings
             JsonObject sendjson = new JsonObject();
-            sendjson.addProperty("productName", configService.getProductName(request.getServerName()));
+            sendjson.addProperty("productName", GuardProductName.getProductName((request.getServerName())));
             sendjson.addProperty("auth", returndata);
             sendjson.addProperty("userid", keys.getUserid());
             sendjson.addProperty("sessionID", sessionID);
             sendjson.addProperty("cid", keys.getContextid());
             sendjson.addProperty("recoveryAvail", keys.isRecoveryAvail());
             sendjson.addProperty("lastMod", keys.getLastup() == null ? "null" : keys.getLastup().toString());
+            sendjson.addProperty("new", keys.getQuestion() == null ? true : false);
             try {
                 //Getting generic guest settings from configuration
                 Settings settings = getGenericGuestPermissions();
@@ -539,8 +576,7 @@ public class Auth {
             GuardLogProperties.put(GuardLogProperties.Name.SESSION_AUTH_ID, returndata);
 
             LOG.info("Guest auth good for " + username);
-
-            response.getWriter().write(sendjson.toString());
+            ServletUtils.sendJsonOK(response, sendjson);
 
         } catch (Exception e) {
             LOG.error("Error during guest login  for user " + username, e);
@@ -616,7 +652,7 @@ public class Auth {
             if (badStorage.isBad(Integer.toString(userid) + "-" + Integer.toString(cid), configService.getIntProperty(GuardProperty.badPasswordCount))) {
                 returnJson.addProperty("auth", "Lockout");
                 returnJson.add("settings", getSettings(null, settings, userid, cid));
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJsonOK(response, returnJson);
                 LOG.info("Lockout auth due to bad attempts");
                 return;
             }
@@ -659,7 +695,7 @@ public class Auth {
             if (keys == null) {
                 returnJson.addProperty("auth", "No Key");
                 returnJson.add("settings", getSettings(null, settings, userid, cid));
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJsonOK(response, returnJson);
                 LOG.debug("Auth sent with No Key for cid = " + cid + " userid = " + userid);
                 return;
             }
@@ -668,7 +704,10 @@ public class Auth {
             returnJson.add("recoveryAvail", new JsonPrimitive(keys.isRecoveryAvail()));
             if (keys.isPasswordNeeded() == true) {
                 returnJson.addProperty("auth", "Password Needed");
-                response.getWriter().write(returnJson.toString());
+                if ((keys.getQuestion() == null) && (keys.isRecoveryAvail())) {  // New, first time use
+                	returnJson.addProperty("new", true);
+                }
+                ServletUtils.sendJson(response, returnJson);
                 LOG.debug("Auth sent with Password Needed for cid = " + cid + " userid = " + userid);
                 return;
             }
@@ -680,12 +719,12 @@ public class Auth {
             }
             if (token == null) {
                 returnJson.addProperty("auth", "Unable to get token");
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJson(response, returnJson);
                 LOG.error("Auth unable to get token from database for cid = " + cid + " userid = " + userid);
                 return;
             }
 
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_GCM);
             try {
                 String oxpassword = ((json == null) || (json.get("ox_password") == null)) ? "" : json.get("ox_password").getAsString();
                 if (oxpassword.equals("null")) {
@@ -742,11 +781,11 @@ public class Auth {
                 }
 
                 returnJson.addProperty("auth", returndata);
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJson(response, returnJson);
             } catch (Exception e) {
                 LOG.error("Error during login", e);
                 returnJson.addProperty("auth", "BAD FORMAT");
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJson(response, returnJson);
                 return;
             }
 
@@ -754,9 +793,12 @@ public class Auth {
             if ((e.getMessage() != null) && (e.getMessage().contains("doesn't exist"))) {
                 returnJson.addProperty("auth", "No Key");
                 returnJson.add("settings", getSettings(null, settings, 0, 0));
-                response.getWriter().write(returnJson.toString());
+                ServletUtils.sendJson(response, returnJson);
             } else {
-                response.getWriter().write("{\"auth\":\"BAD FORMAT\", \"lang\": " + getAvailableLanguagesJson() + "} ");
+            	JsonObject errorJson = new JsonObject();
+            	errorJson.addProperty("auth", "BAD FORMAT");
+            	errorJson.addProperty("lang", getAvailableLanguagesJson().toString());
+            	ServletUtils.sendJsonOK(response, errorJson);
                 LOG.error("Error during login", e);
             }
         }
@@ -764,6 +806,7 @@ public class Auth {
 
     /**
      * Create JSON that gets encrypted and returned as the auth code
+     *
      * @param json
      * @param userid
      * @param cid
@@ -771,14 +814,14 @@ public class Auth {
      * @return
      */
     private JsonObject createAuthJson(JsonObject json, int userid, int cid, String email) {
-    	JsonObject auth = new JsonObject();
-    	auth.addProperty("ox_password", json.has("ox_password") ? json.get("ox_password").getAsString() : "");
-    	auth.addProperty("encr_password", json.has("encr_password") ? json.get("encr_password").getAsString() : "");
-    	auth.addProperty("email", email);
-    	auth.addProperty("user_id", userid);
-    	auth.addProperty("cid", cid);
-    	auth.addProperty("language", json.has("language") ? json.get("language").getAsString() : "");
-    	return (auth);
+        JsonObject auth = new JsonObject();
+        auth.addProperty("ox_password", json.has("ox_password") ? json.get("ox_password").getAsString() : "");
+        auth.addProperty("encr_password", json.has("encr_password") ? json.get("encr_password").getAsString() : "");
+        auth.addProperty("email", email);
+        auth.addProperty("user_id", userid);
+        auth.addProperty("cid", cid);
+        auth.addProperty("language", json.has("language") ? json.get("language").getAsString() : "");
+        return (auth);
     }
 
     private JsonObject getAvailableLanguagesJson() throws OXException {
@@ -982,5 +1025,4 @@ public class Auth {
         return (false);
     }
 
-
 }
