@@ -84,12 +84,15 @@ import com.openexchange.guard.cipher.GuardCipherService;
 import com.openexchange.guard.common.servlets.utils.ServletUtils;
 import com.openexchange.guard.common.util.CipherUtil;
 import com.openexchange.guard.common.util.IDNUtil;
+import com.openexchange.guard.common.util.JsonUtil;
 import com.openexchange.guard.configuration.GuardConfigurationService;
 import com.openexchange.guard.configuration.GuardProperty;
 import com.openexchange.guard.email.storage.ogEmail.Email;
 import com.openexchange.guard.email.storage.ogEmail.EmailStorage;
 import com.openexchange.guard.encryption.EncryptedItem;
 import com.openexchange.guard.encryption.EncryptedItemsStorage;
+import com.openexchange.guard.exceptions.GuardCoreExceptionCodes;
+import com.openexchange.guard.guest.GuestInlineImageHandler;
 import com.openexchange.guard.internal.Auth;
 import com.openexchange.guard.internal.Settings;
 import com.openexchange.guard.internal.Settings.permissions;
@@ -111,10 +114,14 @@ import com.openexchange.guard.oxapi.Utilities;
 import com.openexchange.guard.pgp.PGPMail;
 import com.openexchange.guard.pgp.PGPResult;
 import com.openexchange.guard.pgp.PGPSender;
+import com.openexchange.guard.pgp.PGPUserIdChecker;
 import com.openexchange.guard.pgp.PGPUtils;
 import com.openexchange.guard.pgp.decryption.GuardEMailDecryptionService;
 import com.openexchange.guard.pgp.decryption.PGPDecryptionResult;
 import com.openexchange.guard.pgp.decryption.impl.GuardEMailDecryptionServiceImpl;
+import com.openexchange.guard.pgpcore.services.TokenAuthenticationService;
+import com.openexchange.guard.pgpcore.services.UserIdentity;
+import com.openexchange.guard.ratifier.GuardRatifierService;
 import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 
@@ -131,7 +138,8 @@ public class Incoming {
         sessionId = rq.getParameter("session");
         inlineImageHandlers = new InlineImageHandler[] {
             new AppsuiteInlineImageHandler(),
-            new GuardInlineImageHandler()
+            new GuardInlineImageHandler(),
+            new GuestInlineImageHandler()
         };
     }
 
@@ -191,6 +199,11 @@ public class Incoming {
         }
     }
 
+    private UserIdentity getIdentityForToken(String sessionIdentifier, String authToken) throws OXException {
+        TokenAuthenticationService service = Services.getService(TokenAuthenticationService.class);
+        return service.decryptUserIdentity(sessionIdentifier, authToken);
+    }
+
     /**
      * Handle email sent from UI
      *
@@ -372,7 +385,27 @@ public class Incoming {
                     attach_password = decodeRSApassword(epass);
                 }
             }
+
+            // If password wasn't sent, check if in auth
+            if (attach_password == null && auth != null && !auth.isEmpty()) {
+            	UserIdentity id = getIdentityForToken(cookie.getJSESSIONID(), auth);
+                if (id != null) {
+                    attach_password = new String(id.getPassword());
+                }
+            }
+
             JsonObject data = json.getAsJsonObject("data");
+            JsonArray from;
+            try {
+            	from = data.get("from").getAsJsonArray().get(0).getAsJsonArray();
+            } catch (java.lang.NullPointerException ex) {
+            	throw GuardCoreExceptionCodes.PARAMETER_MISSING.create("From Address Array");
+            }
+            if (!PGPUserIdChecker.checkHasUID(userid, cid, from.get(1).getAsString().trim())) {
+                LOG.info("From email not contained in sender key");
+                ServletUtils.sendNotAcceptable(response, "missingUID");
+                return;
+            }
             ArrayList<RecipKey> recipients = getUsers(data, userid, cid, guestlanguage, pin, info, draft, request.getServerName()); // get the users and populate the recipient keys
             if (recipients == null) {
                 LOG.info("Unable to get keys for all users");
@@ -385,7 +418,7 @@ public class Incoming {
                     if (recipient.isGuest() && recipient.getNewGuestPass() != null && !recipient.getNewGuestPass().isEmpty()) {
                         //Sending a welcome/password email to new guests
                         try {
-                            JsonArray from = data.get("from").getAsJsonArray().get(0).getAsJsonArray();
+
                             String fromName = from.get(0) != JsonNull.INSTANCE ? from.get(0).getAsString().trim() : null;
                             String fromEmail = from.get(1).getAsString().trim();
                             sendInitialGuestPasswordMail(apiAccess, fromName, fromEmail, recipient, templId, guestmessage, request.getServerName(), recipient.getUserid(), recipient.getCid());
@@ -397,9 +430,7 @@ public class Incoming {
                     }
                 }
             }
-
             if (!sendPermission) {
-                JsonArray from = data.get("from").getAsJsonArray().get(0).getAsJsonArray();
                 if (!checkReply(from, recipients, info)) {
                     LOG.info("Attempt to reply to users not in original email list");
                     ServletUtils.sendNotAcceptable(response, "unauth recip");
@@ -492,9 +523,14 @@ public class Incoming {
                 String reply = new PGPSender().sendPGPMail(mail, request.getServerName());
                 if (reply != null)
                 {
-                    ServletUtils.sendOK(response, "application/json", reply);  // If JSON string returned, then return it
+                	if (JsonUtil.isJson(reply)) { // The reply will be either "OK" "Fail" or JSON Information
+                		ServletUtils.sendOK(response, "application/json", reply);  // If JSON string returned, then return it
+                	} else {
+                        ServletUtils.sendOK(response, "text/html", reply);
+                    }
                 }
             } catch (OXException e) {
+            	LOG.error("Problem sending email ", e);
                 ServletUtils.sendNotAcceptable(response, e.getDisplayMessage(new Locale(language)));
             }
 
@@ -553,6 +589,8 @@ public class Incoming {
         for (int i = 0; i < recips.size(); i++) {
             JsonArray recip = recips.get(i).getAsJsonArray();
             String emailString = recip.get(1).getAsString();
+            GuardRatifierService ratifierService = Services.getService(GuardRatifierService.class);
+            ratifierService.validate(emailString); // Will throw error if invalid
             InternetAddress emailAddress = new InternetAddress(IDNUtil.aceEmail(emailString));
             String email = IDNUtil.decodeEmail(emailAddress.getAddress());
 
