@@ -342,7 +342,7 @@ public class PGPPrivateKeyHandler {
         }
         try {
             GuardKeyService guardKeyService = Services.getService(GuardKeyService.class);
-            PrivateKey privateKey = guardKeyService.decryptPrivateKey(existing.getEncodedPrivate(), password, existing.getSalt(), true);
+            PrivateKey privateKey = guardKeyService.decryptPrivateKey(existing.getEncodedPrivate(), password, existing.getSalt());
             String encryptedPK = guardKeyService.encryptPrivateKey(privateKey, password, existing.getSalt());
             newkey.setEncodedPrivate(encryptedPK);
             newkey.setPublic(existing.getPublicKey());
