@@ -209,7 +209,7 @@ public class PGPPublicHandler {
         if (!ring.getPublicKeys().hasNext()) {
             return (false);  // If no public keys, return false
         }
-        JsonArray keydata = PgpHandler.getKeyInfo(ring);
+        JsonArray keydata = PgpHandler.getKeyInfo(ring, false);
         JsonArray ids = new JsonArray();
         List<String> keyhexids = new ArrayList<String>();
         for (int i = 0; i < keydata.size(); i++) {
@@ -343,6 +343,7 @@ public class PGPPublicHandler {
             JsonObject guardJson = getGuardPubKeyDetail(ids);
             if (guardJson != null) {
                 guardJson.addProperty("guard", true);
+                Services.getService(GuardRatifierService.class);
                 ServletUtils.sendHtmlOK(response, guardJson.toString());
                 return;
             }
