@@ -52,6 +52,7 @@ package com.openexchange.guard.pgp;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.common.java.Strings;
+import com.openexchange.guard.keys.internal.hkpclient.RemoteKeyResult;
 
 public class PGPResult {
 
@@ -67,17 +68,20 @@ public class PGPResult {
 
     private String content = null;
 
+    private RemoteKeyResult.Source source = null;
+
     public PGPResult() {
-        this(false, false, null, false, null, null);
+        this(false, false, null, false, null, null, null);
     }
 
-    public PGPResult(boolean signature, boolean verified, byte[] decoded, boolean missingPublicKey, String lContent, String lError) {
+    public PGPResult(boolean signature, boolean verified, byte[] decoded, boolean missingPublicKey, String lContent, String lError, RemoteKeyResult.Source source) {
         this.signature = signature;
         this.verified = verified;
         this.decoded = decoded;
         this.missingPublicKey = missingPublicKey;
         this.content = lContent;
         this.error = lError;
+        this.source = source;
     }
 
     public String getJsonString() {
@@ -91,6 +95,9 @@ public class PGPResult {
         if (Strings.isNotEmpty(this.content)) {
             result.add("content", new JsonPrimitive(this.content));
         }
+        if (source != null) {
+        	result.add("source", new JsonPrimitive(source.getName()));
+        }
         return result.toString();
     }
 
@@ -141,4 +148,12 @@ public class PGPResult {
     public String getContent() {
         return content;
     }
+
+    public void setSource(RemoteKeyResult.Source source) {
+    	this.source = source;
+    }
+
+    public RemoteKeyResult.Source getSource () {
+    	return source;
+    }
 }
