@@ -88,7 +88,7 @@ public class PGPSender {
         // Check if inline requested by a recipient
         List<RecipKey> recipients = mail.getRecipients();
         for (int i = 0; i < recipients.size(); i++) {
-            if (recipients.get(i).reqInline()) {
+        	if (recipients.get(i).isInline()) {
                 lInline = true;
             }
         }
@@ -352,7 +352,7 @@ public class PGPSender {
         String uid = CipherUtil.getUUID();
         String id = "pgp-" + uid;
         for (int i = 0; i < guestRecip.size(); i++) {
-            if (guestRecip.get(i).isGuest() == true) {
+            if (guestRecip.get(i).isGuest() == true && !mail.isDraft()) {
                 saveEmailData(mail, uid, mail.getRecipients(), guestRecip.get(i).getUserid(), guestRecip.get(i).getCid(), fromaddr);
             }
         }
