@@ -99,9 +99,11 @@ import com.openexchange.guard.cipher.GuardCipherFactoryService;
 import com.openexchange.guard.cipher.GuardCipherService;
 import com.openexchange.guard.common.util.LongUtil;
 import com.openexchange.guard.exceptions.GuardAuthExceptionCodes;
+import com.openexchange.guard.keys.HKPClientService;
 import com.openexchange.guard.keys.PGPKeyService;
 import com.openexchange.guard.keys.dao.GuardKeys;
 import com.openexchange.guard.keys.dao.RecipKey;
+import com.openexchange.guard.keys.internal.hkpclient.RemoteKeyResult;
 import com.openexchange.guard.osgi.Services;
 import com.openexchange.guard.pgp.exceptions.GuardPGPExceptionCodes;
 import com.openexchange.guard.session.GuardSessionService;
@@ -124,11 +126,28 @@ public class PGPUtils {
      */
     public static PGPResult decryptFile(InputStream in, int userid, int cid, String password) throws Exception {
         ByteArrayOutputStream fOut = new ByteArrayOutputStream();
-        PGPResult result = decryptFile(in, fOut, userid, cid, password, true);
+        PGPResult result = decryptFile(in, fOut, userid, cid, password, true, null);
         result.setDecoded(fOut.toByteArray());
         return (result);
     }
 
+    /**
+     * Decode PGP item and return decoded in PGPResult
+     * @param in
+     * @param fOut
+     * @param userid
+     * @param cid
+     * @param password
+     * @param close
+     * @return
+     * @throws IOException
+     * @throws OXException
+     * @throws PGPException
+     */
+    public static PGPResult decryptFile(InputStream in, OutputStream fOut, int userid, int cid, String password, boolean close) throws IOException, OXException, PGPException {
+    	return decryptFile (in, fOut, userid, cid, password, close, null);
+    }
+
     /**
      * Decrypt PGP Content
      *
@@ -138,14 +157,19 @@ public class PGPUtils {
      * @param cid
      * @param password The user's password
      * @param close Output stream should be closed and flushed
+     * @param The From address of sender if signature attached
      * @return PGPResult The result
      * @throws IOException
      * @throws OXException
      * @throws PGPException
      */
-    public static PGPResult decryptFile(InputStream in, OutputStream fOut, int userid, int cid, String password, boolean close) throws IOException, OXException, PGPException {
+    public static PGPResult decryptFile(InputStream in, OutputStream fOut, int userid, int cid, String password, boolean close, String fromAddress) throws IOException, OXException, PGPException {
 
-        PGPResult result = new PGPResult();
+        if (password == null || password.isEmpty()) {
+        	LOG.info("Bad password");
+            throw GuardPGPExceptionCodes.BAD_PASSWORD_ERROR.create();
+        }
+    	PGPResult result = new PGPResult();
         in = PGPUtil.getDecoderStream(in);
         PGPObjectFactory pgpF = new PGPObjectFactory(in, new BcKeyFingerprintCalculator());
         PGPEncryptedDataList enc;
@@ -250,7 +274,9 @@ public class PGPUtils {
                 // may not work due to static method
                 PGPKeyService pgpKeyService = Services.getService(PGPKeyService.class);
                 pgpRing = pgpKeyService.getPublicKeyRingById(ops.getKeyID(), userid, cid);
-
+                if (pgpRing == null) {
+                	pgpRing = checkPublicKeys(ops.getKeyID(), fromAddress, result);
+                }
                 result.setSignature(true);
                 if (pgpRing != null) {
                     publicKey = pgpRing.getPublicKey(ops.getKeyID());
@@ -306,7 +332,7 @@ public class PGPUtils {
      * @throws PGPException
      * @throws OXException
      */
-    public PGPResult verifyInlineSignature(InputStream in, String data, int userid, int cid) throws IOException, SignatureException, PGPException, OXException {
+    public PGPResult verifyInlineSignature(InputStream in, String data, int userid, int cid, String fromAddress) throws IOException, SignatureException, PGPException, OXException {
 
         com.openexchange.guard.pgp.PGPResult result = new com.openexchange.guard.pgp.PGPResult();
         in = PGPUtil.getDecoderStream(in);
@@ -327,9 +353,14 @@ public class PGPUtils {
             PGPKeyService pgpKeyService = Services.getService(PGPKeyService.class);
             PGPPublicKeyRing pgpRing = pgpKeyService.getPublicKeyRingById(signature.getKeyID(), userid, cid);
             if (pgpRing == null) {
-                result.setMissingPublicKey(true);
-                result.setError("No public key");
-                return (result);
+            	pgpRing = checkPublicKeys(signature.getKeyID(), fromAddress, result);
+	        	if (pgpRing == null) {
+	                LOG.info("Unable to find pgpRing for sender");
+	                result.setError("No public key");
+	                result.setVerified(false);
+	                result.setMissingPublicKey(true);
+	                return (result);
+	        	}
             }
             PGPPublicKey publicKey = pgpRing.getPublicKey(signature.getKeyID());
             signature.init(new JcaPGPContentVerifierBuilderProvider().setProvider("BC"), publicKey);
@@ -360,7 +391,7 @@ public class PGPUtils {
      * @throws OXException
      * @throws Exception
      */
-    public PGPResult verifySignature(String data, String signature, int userid, int cid) throws IOException, PGPException, OXException {
+    public PGPResult verifySignature(String data, String signature, int userid, int cid, String fromAddress) throws IOException, PGPException, OXException {
         PGPResult result = new PGPResult();
         InputStream datastream = new ByteArrayInputStream(signature.getBytes(StandardCharsets.UTF_8));
         InputStream in = PGPUtil.getDecoderStream(datastream);
@@ -394,11 +425,14 @@ public class PGPUtils {
         PGPKeyService pgpKeyService = Services.getService(PGPKeyService.class);
         PGPPublicKeyRing pgpRing = pgpKeyService.getPublicKeyRingById(sig.getKeyID(), userid, cid);
         if (pgpRing == null) {
-            LOG.info("Unable to find pgpRing for sender");
-            result.setError("No public key");
-            result.setVerified(false);
-            result.setMissingPublicKey(true);
-            return (result);
+        	pgpRing = checkPublicKeys(sig.getKeyID(), fromAddress, result);
+        	if (pgpRing == null) {
+                LOG.info("Unable to find pgpRing for sender");
+                result.setError("No public key");
+                result.setVerified(false);
+                result.setMissingPublicKey(true);
+                return (result);
+        	}
         }
         //	PGPPublicKeyRing pgpRing = gkey.pubring;
 
@@ -426,6 +460,19 @@ public class PGPUtils {
 
     }
 
+    private static PGPPublicKeyRing checkPublicKeys (Long id, String fromEmail, PGPResult result) throws OXException {
+    	HKPClientService hkpClient = Services.getService(HKPClientService.class);
+    	RemoteKeyResult remote = hkpClient.find(null, id, fromEmail);
+    	if (remote.hasResult()) {
+    		if (remote.getSource().isTrusted()) {
+    			result.setSource(remote.getSource());
+    			return remote.getRing();
+    		}
+    	}
+    	return null;
+
+    }
+
     /**
      * Encrypt byte[] data
      *
@@ -508,7 +555,7 @@ public class PGPUtils {
         Iterator it = ring.getPublicKeys();
         while (it.hasNext()) {
             PGPPublicKey key = (PGPPublicKey) it.next();
-            if (key.isEncryptionKey()) {
+            if (com.openexchange.guard.keys.util.PGPUtil.isEncryptionKey(key)) {
                 if (key.isMasterKey()) {  // If master key, we will use only if we don't have another encryption key
                     if (found == null) {
                         found = key;
@@ -548,22 +595,22 @@ public class PGPUtils {
             out = new ArmoredOutputStream(out);
         }
 
-        PGPSecretKey pgpSec = keys.getPGPSigningKey();
-        PGPPrivateKey pgpPrivKey = null;
+        PGPKeyService pgpKeyService = Services.getService(PGPKeyService.class);
+        PGPPrivateKey pgpSignKey;
         try {
-            pgpPrivKey = com.openexchange.guard.keys.util.PGPUtil.decodePrivate(pgpSec, password, keys.getSalt());
-        } catch (PGPException e) {
+            pgpSignKey = pgpKeyService.getPGPSignKey(keys.getPGPSecretKeyRing(), password, keys.getSalt());
+        } catch (OXException ex) {
             throw GuardAuthExceptionCodes.BAD_PASSWORD.create();
         }
-        if (pgpPrivKey == null) {
+        if (pgpSignKey == null) {
             throw GuardPGPExceptionCodes.UNABLE_TO_DECODE.create(keys.getUserid(), keys.getContextid());
         }
+        PGPPublicKey pubkey = keys.getPublicKeyRing().getPublicKey(pgpSignKey.getKeyID());  // Get the public key specific to the signing key
+        PGPSignatureGenerator sGen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(pubkey.getAlgorithm(), PGPUtil.SHA512).setProvider("BC"));
 
-        PGPSignatureGenerator sGen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(pgpSec.getPublicKey().getAlgorithm(), PGPUtil.SHA512).setProvider("BC"));
+        sGen.init(PGPSignature.BINARY_DOCUMENT, pgpSignKey);
 
-        sGen.init(PGPSignature.BINARY_DOCUMENT, pgpPrivKey);
-
-        Iterator it = pgpSec.getPublicKey().getUserIDs();
+        Iterator it = pubkey.getUserIDs();
         if (it.hasNext()) {
             PGPSignatureSubpacketGenerator spGen = new PGPSignatureSubpacketGenerator();
 
@@ -630,7 +677,12 @@ public class PGPUtils {
         int algorithm = com.openexchange.guard.keys.util.PGPUtil.getSigningKey(key.getPGPSecretKeyRing()).getPublicKey().getAlgorithm();
         PGPSignatureGenerator sgen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(algorithm, PGPUtil.SHA512).setProvider("BC"));
         PGPKeyService pgpKeyService = Services.getService(PGPKeyService.class);
-        PGPPrivateKey pgpSignKey = pgpKeyService.getPGPSignKey(key.getPGPSecretKeyRing(), password, key.getSalt());
+        PGPPrivateKey pgpSignKey;
+        try {
+        	pgpSignKey = pgpKeyService.getPGPSignKey(key.getPGPSecretKeyRing(), password, key.getSalt());
+        } catch (OXException ex) {
+        	throw GuardAuthExceptionCodes.BAD_PASSWORD.create();
+        }
 
         sgen.init(PGPSignature.BINARY_DOCUMENT, pgpSignKey);
 
@@ -646,7 +698,7 @@ public class PGPUtils {
     }
 
     public static String encodeEPass(String password, String jSessionId, int userid, int cid) throws Exception {
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         GuardSessionService sessionService = Services.getService(GuardSessionService.class);
         String token = sessionService.getToken(jSessionId);
         if (token == null) {
@@ -658,7 +710,7 @@ public class PGPUtils {
     public static String decodeEPass(String epass, String sessionId) throws OXException {
         GuardSessionService sessionService = Services.getService(GuardSessionService.class);
         String token = sessionService.getToken(sessionId);
-        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+        GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
         return (cipherService.decrypt(epass, token));
     }
 
