@@ -58,9 +58,12 @@ import java.net.URLEncoder;
 import java.security.NoSuchProviderException;
 import java.util.ArrayList;
 import java.util.Properties;
+
+import javax.mail.Address;
 import javax.mail.MessagingException;
 import javax.mail.Part;
 import javax.mail.Session;
+import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MimeMessage;
 import org.apache.commons.io.IOUtils;
 import org.apache.http.entity.ContentType;
@@ -69,6 +72,10 @@ import org.apache.http.entity.mime.MultipartEntity;
 import org.apache.http.entity.mime.content.ByteArrayBody;
 import org.apache.http.entity.mime.content.StringBody;
 import org.bouncycastle.openpgp.PGPException;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.openexchange.exception.OXException;
 import com.openexchange.guard.bad.GuardBadStorage;
@@ -99,6 +106,8 @@ import com.openexchange.guard.pgp.exceptions.GuardPGPExceptionCodes;
  */
 public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionService {
 
+	private static Logger LOG = LoggerFactory.getLogger(GuardEMailDecryptionServiceImpl.class);
+
     /**
      * Retrieves the mime email from the OX backend
      *
@@ -133,15 +142,51 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
      * @return
      * @throws UnsupportedEncodingException
      */
-    private String getInlineEMailContent(OxCookie cookie, String session, String userAgent, String emailFolder, String emailId, int attachmentId) throws UnsupportedEncodingException {
-        String jsonString = new Api(cookie, session, userAgent).getMail(emailId, emailFolder);
-        JsonObject json = JsonUtil.parseAsJsonObject(jsonString);
+    private String getInlineEMailContent(JsonObject json, int attachmentId) throws UnsupportedEncodingException {
         JsonObject data = json.get("data").getAsJsonObject();
         JsonObject att = data.get("attachments").getAsJsonArray().get(attachmentId).getAsJsonObject();
         String content = att.get("content").getAsString();
+        // Remove surrounding HTML
+        int i = content.indexOf("-----BEGIN PGP MESSAGE");
+        if (i > 0) {
+        	content = content.substring(i);
+        }
+        int j = content.indexOf("-----END PGP MESSAGE");
+        if (j > 0) {
+        	int k = content.indexOf("\r", j);
+        	if (k > 0) {
+        		content = content.substring(0, k);
+        	}
+        }
+        // Convert <br> to "\r", but not if \r already there
+        content = content.replace("\r<br>", "\r").replace("<br>\r", "\r").replace("<br>", "\r");
+        StringBuilder sb = new StringBuilder();
+        char[] chars = content.toCharArray();
+        // Remove non-ascii characters (sometimes appear in body due to encoding)
+        for (char c : chars) {
+        	if (c < 0x7f) {
+        		sb.append(c);
+        	}
+        }
+        return sb.toString();
+    }
 
-        content = content.replace("<br>", "\r\n");
-        return content;
+    private String getInlineFrom (JsonObject json) {
+    	String address = null;
+    	try {
+	    	JsonObject data = json.get("data").getAsJsonObject();
+	    	if (data.get("from").isJsonArray()) {
+	    		JsonArray fromArray = data.get("from").getAsJsonArray();
+	    		address = fromArray.get(0).getAsJsonArray().get(1).getAsString().trim();
+	    	}
+	    	if (data.get("from").isJsonObject()) {
+	    		address = data.get("from").getAsJsonArray().get(1).getAsString().trim();
+	    	}
+	    	return address;
+    	} catch (Exception ex) {
+    		LOG.error("Problem parsing inline message for FROM address ", ex);
+    	}
+    	return address;
     }
 
     //FIXME/TODO: this method should be placed in another class
@@ -199,7 +244,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
         String response = api.filePost(json.toString(), params, entity);
         return JsonUtil.extractObject(response);
     }
-    
+
 
     /**
     * TODO Refactor and centralize BF handling
@@ -213,7 +258,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
            throw GuardBruteForceDetectionExceptionCodes.ACCOUNT_LOCKED.create();
        }
    }
-   
+
    /**
     * TODO Refactor and centralize BF handling
     * Adds a failed login attempt for a given id
@@ -225,22 +270,31 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
        badService.addBad(id);
    }
 
+   private String getFrom (MimeMessage msg) throws MessagingException {
+	   Address[] fromArray = msg.getFrom();
+	   if (fromArray.length > 0) {
+		   InternetAddress from = new InternetAddress(fromArray[0].toString());
+		   return from.getAddress();
+	   }
+	   return "";
+   }
+
     @Override
     public PGPDecryptionResult decryptMimeEmail(OxCookie cookie, String session, String userAgent, String emailFolder, String emailId, int userId, int contextId, String password) throws OXException {
 
         try {
             checkForBruteForce(cookie.getJSESSIONID());
-            
+
             //Getting the encrypted mime email
             MimeMessage message = getMimeEMail(cookie, session, userAgent, emailFolder, emailId);
 
             //De-Crypting the mime message
             try (ByteArrayOutputStream decryptedMessage = new ByteArrayOutputStream()) {
                 PGPMimeDecrypter decrypter = new PGPMimeDecrypter();
-                PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage);
+                PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage, getFrom(message));
 
                 try (InputStream decryptedMessageStream = new ByteArrayInputStream(decryptedMessage.toByteArray())) {
-                    return new PGPDecryptionResult(decryptedMessageStream, message.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.getError());
+                    return new PGPDecryptionResult(decryptedMessageStream, message.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.isMissingPublicKey(), pgpResult.getError());
                 }
             }
         }
@@ -259,15 +313,18 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
     public PGPDecryptionResult decryptInlineEmail(OxCookie cookie, String session, String userAgent, String emailFolder, String emailId, int attachmentId, int userId, int contextId, String password) throws OXException {
         try {
             checkForBruteForce(cookie.getJSESSIONID());
-            
-            //Getting the Inline Email
-            String inlineEMailContent = getInlineEMailContent(cookie, session, userAgent, emailFolder, emailId, attachmentId);
 
+            //Getting the Inline Email
+            String jsonString = new Api(cookie, session, userAgent).getMail(emailId, emailFolder);
+            JsonObject json = JsonUtil.parseAsJsonObject(jsonString);
+
+            String inlineEMailContent = getInlineEMailContent(json, attachmentId);
+            String fromAddress = getInlineFrom(json);
             //Decrypt and return the result
             try (ByteArrayOutputStream decryptedMessage = new ByteArrayOutputStream()) {
-                PGPResult pgpResult = new PGPInlineDecrypter().decryptMessage(inlineEMailContent, userId, contextId, password, decryptedMessage);
+                PGPResult pgpResult = new PGPInlineDecrypter().decryptMessage(inlineEMailContent, userId, contextId, password, decryptedMessage, fromAddress);
                 try (InputStream decryptedMessageStream = new ByteArrayInputStream(decryptedMessage.toByteArray())) {
-                    return new PGPDecryptionResult(decryptedMessageStream, null, pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.getError());
+                    return new PGPDecryptionResult(decryptedMessageStream, null, pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.isMissingPublicKey(), pgpResult.getError());
                 }
             }
         }
@@ -292,14 +349,14 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
 
         try {
             checkForBruteForce(cookie.getJSESSIONID());
-            
+
             //Getting the encrypted mime email
             MimeMessage message = getMimeEMail(cookie, session, userAgent, emailFolder, emailId);
 
             //De-Crypting the mime message
             ByteArrayOutputStream decryptedMessage = new ByteArrayOutputStream();
             PGPMimeDecrypter decrypter = new PGPMimeDecrypter();
-            PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage);
+            PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage, getFrom(message));
 
             //Extracting the attachment from the decrypted mime message by it's name
             try (InputStream decryptedMessageStream = new ByteArrayInputStream(decryptedMessage.toByteArray())) {
@@ -307,7 +364,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
                 if (decryptedAttachment == null) {
                     throw GuardPGPExceptionCodes.MIME_ATTACHMENT_NOT_FOUND.create(attachmentName);
                 }
-                return new PGPDecryptionResult(decryptedAttachment.getInputStream(), decryptedAttachment.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.getError());
+                return new PGPDecryptionResult(decryptedAttachment.getInputStream(), decryptedAttachment.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.isMissingPublicKey(), pgpResult.getError());
             }
         }
         catch(OXException e) {
@@ -325,23 +382,23 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
     public PGPDecryptionResult decryptMimeAttachmentById(OxCookie cookie, String session, String userAgent, String emailFolder, String emailId, String attachmentId, int userId, int contextId, String password) throws OXException {
         try {
             checkForBruteForce(emailFolder + "/" + emailId + "a:" + attachmentId);
-            
+
             //Getting the encrypted mime email
             MimeMessage message = getMimeEMail(cookie, session, userAgent, emailFolder, emailId);
 
             //De-Crypting the mime message
             ByteArrayOutputStream decryptedMessage = new ByteArrayOutputStream();
             PGPMimeDecrypter decrypter = new PGPMimeDecrypter();
-            PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage);
+            PGPResult pgpResult = decrypter.decryptMessage(message, userId, contextId, password, decryptedMessage, getFrom(message));
 
             //Extracting the attachment from the decrypted mime message by ID
             try (InputStream decryptedMessageStream = new ByteArrayInputStream(decryptedMessage.toByteArray())) {
-                Part decryptedAttachment = new PGPMimeAttachmentExtractor().getAttachmentFromMessageID(decryptedMessageStream, attachmentId);                
+                Part decryptedAttachment = new PGPMimeAttachmentExtractor().getAttachmentFromMessageID(decryptedMessageStream, attachmentId);
                 if (decryptedAttachment == null) {
                     throw GuardPGPExceptionCodes.MIME_ATTACHMENT_ID_NOT_FOUND.create(attachmentId);
                 }
                 String ct = decryptedAttachment.getContentType();
-                return new PGPDecryptionResult(decryptedAttachment.getInputStream(),decryptedAttachment.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.getError());
+                return new PGPDecryptionResult(decryptedAttachment.getInputStream(),decryptedAttachment.getContentType(), pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.isMissingPublicKey(), pgpResult.getError());
             }
         }
         catch(OXException e) {
@@ -365,7 +422,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
 
         try {
             checkForBruteForce(emailFolder + "/" + emailId + "a:" + attachmentId);
-            
+
             //Get the encrypted attachment
             Api api = new Api(cookie, session, userAgent);
             Attachment att = new Attachment();
@@ -377,19 +434,19 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
                     contentType = response.getContentType().getValue();
                 }
             }
-            
+
             try (InputStream attachmentStream = new ByteArrayInputStream(att.getContent()); ByteArrayOutputStream decryptedAttachment = new ByteArrayOutputStream()) {
                 //Decrypt email attachment and return an InputStream to it
                 PGPResult pgpResult = PGPUtils.decryptFile(attachmentStream, decryptedAttachment, userId, contextId, password, false/* do not close the stream */);
-                
-                return new PGPDecryptionResult(new ByteArrayInputStream(decryptedAttachment.toByteArray()), contentType, pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.getError());
+
+                return new PGPDecryptionResult(new ByteArrayInputStream(decryptedAttachment.toByteArray()), contentType, pgpResult.isSignature(), pgpResult.isVerified(), pgpResult.isMissingPublicKey(), pgpResult.getError());
             }
-        } 
+        }
         catch (OXException e) {
             if(e.getExceptionCode() == GuardPGPExceptionCodes.BAD_PASSWORD_ERROR) {
                 countBruteForceAttempt(emailFolder + "/" + emailId + "a:" + attachmentId);
             }
-            throw e;            
+            throw e;
         }
         catch (IOException | PGPException e) {
             throw GuardPGPExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
@@ -401,7 +458,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
     public JsonObject decryptMimeAttachmentToDrive(OxCookie cookie, String session, String userAgent, String emailFolder, String emailId, String attachmentName, int folderId, String fileName, String description, int userId, int contextId, String password, AttachmentSaveMode saveMode) throws OXException {
         try {
             checkForBruteForce(emailFolder + "/" + emailId + "a:" + attachmentName);
-            
+
             //Decrypt the mime attachment
             PGPDecryptionResult decryptionResult = decryptMimeAttachmentByName(cookie, session, userAgent, emailFolder, emailId, attachmentName, userId, contextId, password);
             //Save it as new item into drive
@@ -412,7 +469,7 @@ public class GuardEMailDecryptionServiceImpl implements GuardEMailDecryptionServ
             if(e.getExceptionCode() == GuardPGPExceptionCodes.BAD_PASSWORD_ERROR) {
                 countBruteForceAttempt(emailFolder + "/" + emailId + "a:" + attachmentName);
             }
-            throw e;            
+            throw e;
         }
         catch (NoSuchProviderException | IOException | PGPException e) {
             throw GuardPGPExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
