@@ -66,6 +66,7 @@ import com.openexchange.guard.servlets.login.LoginServlet;
 import com.openexchange.guard.servlets.mail.MailServlet;
 import com.openexchange.guard.servlets.mailvelope.MailvelopeServlet;
 import com.openexchange.guard.servlets.pgp.PGPServlet;
+import com.openexchange.guard.servlets.pgpcore.PGPCoreServlet;
 import com.openexchange.guard.servlets.pgpmail.PGPMailServlet;
 import com.openexchange.guard.servlets.ping.PingServlet;
 import com.openexchange.guard.servlets.settings.SettingsServlet;
@@ -83,6 +84,7 @@ public class GuardServlets {
 
     private static final String GUARD_LOGIN_SERVLET_PATH = "/oxguard/login";
     private static final String GUARD_PING_SERVLET_PATH = "/oxguard/ping";
+    private static final String GUARD_PGP_CORE_SERVLET_PATH = "/oxguard/pgpcore";
     private static final String GUARD_PGP_SERVLET_PATH = "/oxguard/pgp";
     private static final String GUARD_PGPMAIL_SERVLET_PATH = "/oxguard/pgpmail";;
     private static final String GUARD_MAIL_SERVLET_PATH = "/oxguard/mail";;
@@ -111,6 +113,7 @@ public class GuardServlets {
             return;
         }
         httpService.registerServlet(GUARD_PING_SERVLET_PATH, new PingServlet(), null, null);
+        httpService.registerServlet(GUARD_PGP_CORE_SERVLET_PATH, new PGPCoreServlet(), null, null);
         httpService.registerServlet(GUARD_PGP_SERVLET_PATH, new PGPServlet(), null, null);
         httpService.registerServlet(GUARD_PGPMAIL_SERVLET_PATH, new PGPMailServlet(), null, null);
         httpService.registerServlet(GUARD_MAIL_SERVLET_PATH, new MailServlet(), null, null);
@@ -138,6 +141,7 @@ public class GuardServlets {
             return;
         }
         httpService.unregister(GUARD_PING_SERVLET_PATH);
+        httpService.unregister(GUARD_PGP_CORE_SERVLET_PATH);
         httpService.unregister(GUARD_PGP_SERVLET_PATH);
         httpService.unregister(GUARD_PGPMAIL_SERVLET_PATH);
         httpService.unregister(GUARD_MAIL_SERVLET_PATH);
