@@ -279,7 +279,7 @@ public class Guest {
         if (password != null) {
             GuardSessionService sessionService = Services.getService(GuardSessionService.class);
             String token = sessionService.getToken(cookie.getJSESSIONID());
-            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+            GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
             Results.addProperty("epassword", cipherService.encrypt(password, token));
         }
         if (result.getError() != null) {
@@ -447,7 +447,7 @@ public class Guest {
                 }
             }
             if (prevanswer != null) {
-                GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES);
+                GuardCipherService cipherService = Services.getService(GuardCipherFactoryService.class).getCipherService(GuardCipherAlgorithm.AES_CBC);
                 if (Strings.isEmpty(question)) {
                     question = "default";
                 }
