@@ -92,7 +92,7 @@ public class LoginServlet extends AbstractGuardServlet {
         servletPostActions = new HashMap<String, GuardServletAction>();
         servletPostActions.put(LOGIN_ACTION, new LoginAction().setAuthenticationHandler(uiSessionAuthentication));
         servletPostActions.put(LOGIN_GUEST_ACTION, new LoginGuestAction());
-        servletPostActions.put(CHANGE_PASSWORD_ACTION, new ChangePasswordAction());
+        servletPostActions.put(CHANGE_PASSWORD_ACTION, new ChangePasswordAction().setAuthenticationHandler(uiSessionAuthentication));
         servletPostActions.put(CREATE_KEYS_ACTION, new CreateKeysActions().setAuthenticationHandler(uiSessionAuthentication));
         servletPostActions.put(CHANGE_SECONDARY_EMAIL_ACTION, new ChangeSecondaryEmailAction().setAuthenticationHandler(uiSessionAuthentication));
         servletPostActions.put(DELETE_RECOVERY_ACTION, new DeleteRecoveryAction().setAuthenticationHandler(uiSessionAuthentication));
