@@ -88,14 +88,14 @@ public class UserDeleter {
      * @throws OXException due an error
      */
     public void deleteWithBackup(int userId, int contextId) throws OXException {
-
+        
         KeyTableStorage keyTableStorage = Services.getService(KeyTableStorage.class);
-
-        List<GuardKeys> keys = keyTableStorage.getKeysForUser(userId, contextId);
-        backupKeys(keys);
-
-        removeKeyMappings(keys);
-        removeKeys(userId, contextId);
+        if (keyTableStorage.exists(userId, contextId, 0 /* not relevant */)) {
+            List<GuardKeys> keys = keyTableStorage.getKeysForUser(userId, contextId);
+            backupKeys(keys);
+            removeKeyMappings(keys);
+            removeKeys(userId, contextId);
+        }
         //The order matters: removing the email mapping at the end, because it contains the shard id for the other operations
         removeEmailMappings(userId, contextId);
     }
@@ -111,10 +111,11 @@ public class UserDeleter {
     public void delete(int userId, int contextId) throws OXException {
 
         KeyTableStorage keyTableStorage = Services.getService(KeyTableStorage.class);
-
-        List<GuardKeys> keys = keyTableStorage.getKeysForUser(userId, contextId);
-        removeKeyMappings(keys);
-        removeKeys(userId, contextId);
+        if (keyTableStorage.exists(userId, contextId, 0 /* not relevant */)) {
+            List<GuardKeys> keys = keyTableStorage.getKeysForUser(userId, contextId);
+            removeKeyMappings(keys);
+            removeKeys(userId, contextId);
+        }
         //The order matters: removing the email mapping at the end, because it contains the shard id for the other operations
         removeEmailMappings(userId, contextId);
     }
